@@ -3,7 +3,7 @@ import os
 import re
 import socket
 import zlib
-import imp
+import importlib
 import subprocess as ssubprocess
 import shlex
 from shlex import quote
@@ -14,43 +14,15 @@ import sshuttle.helpers as helpers
 from sshuttle.helpers import debug2
 
 
-def readfile(name):
-    tokens = name.split(".")
-    f = None
-
-    token = tokens[0]
-    token_name = [token]
-    token_str = ".".join(token_name)
-
-    try:
-        f, pathname, description = imp.find_module(token_str)
-
-        for token in tokens[1:]:
-            module = imp.load_module(token_str, f, pathname, description)
-            if f is not None:
-                f.close()
-
-            token_name.append(token)
-            token_str = ".".join(token_name)
-
-            f, pathname, description = imp.find_module(
-                token, module.__path__)
-
-        if f is not None:
-            contents = f.read()
-        else:
-            contents = ""
-
-    finally:
-        if f is not None:
-            f.close()
-
-    return contents.encode("UTF8")
+def get_module_source(name):
+    spec = importlib.util.find_spec(name)
+    with open(spec.origin, "rt") as f:
+        return f.read().encode("utf-8")
 
 
 def empackage(z, name, data=None):
     if not data:
-        data = readfile(name)
+        data = get_module_source(name)
     content = z.compress(data)
     content += z.flush(zlib.Z_SYNC_FLUSH)
 
@@ -68,17 +40,17 @@ def parse_hostport(rhostport):
 
     and returns a tuple (username, password, port, host)
     """
-    # default port for SSH is TCP port 22
-    port = 22
+    # leave use of default port to ssh command to prevent overwriting
+    # ports configured in ~/.ssh/config when no port is given
+    port = None
     username = None
     password = None
     host = rhostport
 
     if "@" in host:
         # split username (and possible password) from the host[:port]
-        username, host = host.split("@")
+        username, host = host.rsplit("@", 1)
         # Fix #410 bad username error detect
-        # username cannot contain an @ sign in this scenario
         if ":" in username:
             # this will even allow for the username to be empty
             username, password = username.split(":")
@@ -117,7 +89,7 @@ def connect(ssh_cmd, rhostport, python, stderr, options):
         rhost = host
 
     z = zlib.compressobj(1)
-    content = readfile('sshuttle.assembler')
+    content = get_module_source('sshuttle.assembler')
     optdata = ''.join("%s=%r\n" % (k, v) for (k, v) in list(options.items()))
     optdata = optdata.encode("UTF8")
     content2 = (empackage(z, 'sshuttle') +
@@ -145,6 +117,10 @@ def connect(ssh_cmd, rhostport, python, stderr, options):
             sshl = shlex.split(ssh_cmd)
         else:
             sshl = ['ssh']
+        if port is not None:
+            portl = ["-p", str(port)]
+        else:
+            portl = []
         if python:
             pycmd = "'%s' -c '%s'" % (python, pyscript)
         else:
@@ -155,12 +131,12 @@ def connect(ssh_cmd, rhostport, python, stderr, options):
         if password is not None:
             os.environ['SSHPASS'] = str(password)
             argv = (["sshpass", "-e"] + sshl +
-                    ["-p", str(port)] +
+                    portl +
                     [rhost, '--', pycmd])
 
         else:
             argv = (sshl +
-                    ["-p", str(port)] +
+                    portl +
                     [rhost, '--', pycmd])
     (s1, s2) = socket.socketpair()
 
